==> webkitgtk-2.24.0.tar.xz: webkitgtk-2.24.0/Source/WebCore/Modules/webgpu/WHLSL/WHLSLHighZombieFinder.cpp <==
/*
 * Copyright (C) 2019 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "WHLSLHighZombieFinder.h"

#if ENABLE(WEBGPU)

#include "WHLSLVisitor.h"

namespace WebCore {

namespace WHLSL {

#if !ASSERT_DISABLED
// If a high-level construct somehow manages to live on when we're lowered, it's a high zombie.
class HighZombieFinder : public Visitor {
private:
    void visit(AST::DotExpression&) override
    {
        ASSERT_NOT_REACHED();
    }

    void visit(AST::IndexExpression&) override
    {
        ASSERT_NOT_REACHED();
    }

    void visit(AST::ReadModifyWriteExpression&) override
    {
        ASSERT_NOT_REACHED();
    }
};
#endif

void findHighZombies(Program& program)
{
#if ASSERT_DISABLED
    UNUSED_PARAM(program);
#else
    HighZombieFinder().Visitor::visit(program);
#endif
}

}

}

#endif

==> webkitgtk-2.27.2.tar.xz: webkitgtk-2.27.2/Source/WebCore/Modules/webgpu/WHLSL/WHLSLHighZombieFinder.cpp <==
/*
 * Copyright (C) 2019 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "WHLSLHighZombieFinder.h"

#if ENABLE(WEBGPU)

#include "WHLSLAST.h"
#include "WHLSLVisitor.h"

namespace WebCore {

namespace WHLSL {

class HighZombieFinder : public Visitor {
public:
    void visit(AST::ReadModifyWriteExpression& readModifyWrite) override
    {
        checkErrorAndVisit(readModifyWrite.newValueExpression());
        checkErrorAndVisit(readModifyWrite.resultExpression());

        RELEASE_ASSERT(!readModifyWrite.leftValue().mayBeEffectful());
    }

    void visit(AST::DotExpression& dotExpression) override
    {
        RELEASE_ASSERT(!dotExpression.base().mayBeEffectful());
    }

    void visit(AST::IndexExpression& indexExpression) override
    {
        RELEASE_ASSERT(!indexExpression.base().mayBeEffectful());
        RELEASE_ASSERT(!indexExpression.indexExpression().mayBeEffectful());
    }
};

void findHighZombies(Program& program)
{
    UNUSED_PARAM(program);

#if !ASSERT_DISABLED
    HighZombieFinder finder;
    finder.Visitor::visit(program);
#endif
}

}

}

#endif
